@@ -9,7 +9,7 @@
 //  Module:         Standard include file
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -76,13 +76,15 @@ extern void *pOffsetOfNullPointer;
 #define OFFSET_OF(Type, Element)                            \
    ((unsigned int) &(((Type*)pOffsetOfNullPointer)->Element))
 
-#endif
-
 
 class   t_Device;                    // As t_Device is the core structure of guymager and as it is needed
 typedef t_Device       *t_pDevice;   // all the time, it is easiest to declare it here (eventhough I don't
 typedef t_Device const *t_pcDevice;  // like that style too much, but we won't change C++ at this time).
 
+class   t_Acquisition;
+typedef t_Acquisition       *t_pAcquisition;
+typedef t_Acquisition const *t_pcAcquisition;
+
 #define EWF_MULTITHREADED_COMPRESSION_CHUNK_SIZE       32768
 
 const unsigned long long BYTES_PER_MiB = 1ULL << 20;
@@ -109,3 +111,4 @@ typedef short              t_int16;
 typedef int                t_int32;
 typedef long long          t_int64;
 
+#endif
