@@ -9,7 +9,7 @@
 //  Module:         Dialog that is opened when aborting an acquisition
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -26,9 +26,10 @@
 // You should have received a copy of the GNU General Public License
 // along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
-#include <QtGui>
 
 #include "dlgabort.h"
+#include "config.h"
+#include "qtutil.h"
 
 // -----------------------------
 //           Constants
@@ -63,10 +64,14 @@ t_DlgAbort::t_DlgAbort (t_pcDevice pDevice, QWidget *pParent, Qt::WFlags Flags)
       Initialised = true;
    }
 
-   pOwn = new t_DlgAbortLocal;
+   QTUTIL_SET_FONT (this, FONTOBJECT_MESSAGE_DIALOGS)
 
-   Acquisition = ((pDevice->State == t_Device::Acquire) ||
-                  (pDevice->State == t_Device::AcquirePaused));
+   pOwn = new t_DlgAbortLocal;
+   pOwn->pCheckBoxDelete = NULL;
+
+   Acquisition = ((pDevice->GetState() == t_Device::Acquire)       ||
+                  (pDevice->GetState() == t_Device::AcquirePaused) ||
+                  (pDevice->GetState() == t_Device::Queued));
    if (Acquisition)
         Text = tr ("Do you want to abort the acquisition of %1?" ) .arg(pDevice->LinuxDevice);
    else Text = tr ("Do you want to abort the verification of %1?") .arg(pDevice->LinuxDevice);
@@ -76,14 +81,14 @@ t_DlgAbort::t_DlgAbort (t_pcDevice pDevice, QWidget *pParent, Qt::WFlags Flags)
    QLabel      *pLabel       = new QLabel      (Text, this);
    QPushButton *pButtonOk    = new QPushButton (QObject::tr("Ok"    ), this);
    QPushButton *pButtonCancel= new QPushButton (QObject::tr("Cancel"), this);
-   if (!pDevice->Acquisition.Clone)
+   if (!pDevice->Acquisition1.Clone)
    {
       pOwn->pCheckBoxDelete = new QCheckBox (tr("Delete partial image files"), this);
       pOwn->pCheckBoxDelete->setChecked (Acquisition);
    }
 
    pMainLayout->addWidget   (pLabel);
-   if (!pDevice->Acquisition.Clone)
+   if (pOwn->pCheckBoxDelete)
       pMainLayout->addWidget   (pOwn->pCheckBoxDelete);
    pMainLayout->addLayout   (pButtonLayout);
    pButtonLayout->addWidget (pButtonOk);
@@ -112,9 +117,9 @@ APIRET t_DlgAbort::Show (t_pcDevice pDevice, bool &Abort, bool &Delete)
    pDlg->setModal  (true);
    Result = pDlg->exec();
    Abort  = (Result == QDialog::Accepted);
-   if (pDevice->Acquisition.Clone)
-        Delete = false;
-   else Delete = (Abort && pDlg->pOwn->pCheckBoxDelete->isChecked());
+   if (pDlg->pOwn->pCheckBoxDelete)
+        Delete = (Abort && pDlg->pOwn->pCheckBoxDelete->isChecked());
+   else Delete = false;
    delete pDlg;
 
    return NO_ERROR;
