@@ -983,8 +983,9 @@ void t_DlgAcquire::UpdateFieldState (void)
 
    // EWF fields
    // ----------
-   Enabled = (pOwn->pRadioButtonFormatEWF->isChecked() ||
-              pOwn->pRadioButtonFormatAFF->isChecked());
+   Enabled =  pOwn->pRadioButtonFormatEWF->isChecked() ||
+              pOwn->pRadioButtonFormatAFF->isChecked() ||
+             (pOwn->pRadioButtonFormatDD ->isChecked() && CONFIG (InfoFieldsForDd));
    for (i = 0;
         i < pOwn->EwfWidgetsList.count();
         i++)
@@ -1245,7 +1246,7 @@ void t_DlgAcquire::Accept (bool DuplicatePressed)
             return;
          }
          SplitSize = NumValue * UnitIndexToMultiplier (DlgAcquireGetField(CFG_DLGACQUIRE_SPLITFILEUNIT)->pComboBox->currentIndex());
-         #if ((LIBEWF_VERSION != 20100226) && (LIBEWF_VERSION != 20080501) && (LIBEWF_VERSION != 20111015) && (LIBEWF_VERSION != 20130416))
+         #if ((LIBEWF_VERSION != 20080501) && (LIBEWF_VERSION != 20100226) && (LIBEWF_VERSION != 20111015) && (LIBEWF_VERSION < 20130416))
             #error "Please check EWF documentation for newer Encase formats and adjust following code"
          #endif
          if (pOwn->pRadioButtonFormatEWF->isChecked())
