@@ -9,7 +9,7 @@
 //  Module:         Wait dialog
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -27,6 +27,8 @@
 // along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
 #include "dlgwait.h"
+#include "config.h"
+#include "qtutil.h"
 
 // -----------------------------
 //           Constants
@@ -61,6 +63,8 @@ t_DlgWait::t_DlgWait (const QString &Title, const QString &Message, QWidget *pPa
       CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_DLGWAIT_CONSTRUCTOR_NOT_SUPPORTED))
    }
 
+   QTUTIL_SET_FONT (this, FONTOBJECT_MESSAGE_DIALOGS)
+
    pOwn = new t_DlgWaitLocal;
 
    pLayout = new QVBoxLayout ();
