@@ -10,7 +10,7 @@
 //                  etc
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -53,8 +53,8 @@ namespace t_File
 
    APIRET GetFormatDescription (t_Format Format, bool Clone, int DdSplitDecimals, QString &Str);
    APIRET GetFormatExtension   (t_Format Format, bool Clone, int DdSplitDecimals, QString *pExtWildcards, QString *pExtHumanReadable=NULL);
-   APIRET GetFormatDescription (t_pDevice pDevice, QString &Str);
-   APIRET GetFormatExtension   (t_pDevice pDevice, QString *pExtWildCards, QString *pExtHumanReadable=NULL);
+   APIRET GetFormatDescription (t_pcDevice pDevice, QString &Str);
+   APIRET GetFormatExtension   (t_pcDevice pDevice, QString *pExtWildCards, QString *pExtHumanReadable=NULL);
 
    APIRET Init                 (void);
 }
