@@ -9,7 +9,7 @@
 //  Module:         Hash wrapper functions for uniform hash interface
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -35,7 +35,7 @@
 #include <openssl/md5.h>
 #endif
 
-#ifdef USE_SHA256_FROM_OPENSSL
+#if defined(USE_SHA256_FROM_OPENSSL) || defined(USE_SHA1_FROM_OPENSSL)
 #include <openssl/sha.h>
 #endif
 
@@ -90,11 +90,65 @@ APIRET HashMD5DigestStr (t_pHashMD5Digest pDigest, QString &Str)
    return NO_ERROR;
 }
 
-bool HashMD5Match (t_pHashMD5Digest pDigest1, t_pHashMD5Digest pDigest2)
+bool HashMD5Match (t_pcHashMD5Digest pDigest1, t_pcHashMD5Digest pDigest2)
 {
    return (memcmp (pDigest1, pDigest2, sizeof(t_HashMD5Digest)) == 0);
 }
 
+// ------------------------------------
+//           SHA1 Functions
+// ------------------------------------
+
+APIRET HashSHA1Clear (t_pHashContextSHA1 pContext)
+{
+   memset (pContext, 0, sizeof(t_HashContextSHA1));
+   return NO_ERROR;
+}
+
+APIRET HashSHA1Init (t_pHashContextSHA1 pContext)
+{
+   #ifdef USE_SHA1_FROM_OPENSSL
+      (void) SHA1_Init (pContext);
+   #else
+      SHA1Init (pContext);
+   #endif
+   return NO_ERROR;
+}
+
+APIRET HashSHA1Append (t_pHashContextSHA1 pContext, void *pData, int DataLen)
+{
+   #ifdef USE_SHA1_FROM_OPENSSL
+      (void) SHA1_Update (pContext, pData, (unsigned long) DataLen);
+   #else
+      SHA1Append (pContext, (unsigned char *)pData, DataLen);
+   #endif
+   return NO_ERROR;
+}
+
+APIRET HashSHA1Digest (t_pHashContextSHA1 pContext, t_pHashSHA1Digest pDigest)
+{
+   #ifdef USE_SHA1_FROM_OPENSSL
+      (void) SHA1_Final (&pDigest->Buff[0], pContext);
+   #else
+      SHA1Finish (pContext, (unsigned char *)pDigest);
+   #endif
+   return NO_ERROR;
+}
+
+APIRET HashSHA1DigestStr (t_pHashSHA1Digest pDigest, QString &Str)
+{
+   Str.clear();
+   for (int i=0; i<HASH_SHA1_DIGEST_LENGTH; i++)
+      Str += QString ("%1") .arg ((int)(pDigest->Buff[i]), 2, 16, QChar('0'));
+   return NO_ERROR;
+}
+
+bool HashSHA1Match (t_pcHashSHA1Digest pDigest1, t_pcHashSHA1Digest pDigest2)
+{
+   return (memcmp (pDigest1, pDigest2, sizeof(t_HashSHA1Digest)) == 0);
+}
+
+
 // ------------------------------------
 //           SHA256 Functions
 // ------------------------------------
@@ -143,7 +197,7 @@ APIRET HashSHA256DigestStr (t_pHashSHA256Digest pDigest, QString &Str)
    return NO_ERROR;
 }
 
-bool HashSHA256Match (t_pHashSHA256Digest pDigest1, t_pHashSHA256Digest pDigest2)
+bool HashSHA256Match (t_pcHashSHA256Digest pDigest1, t_pcHashSHA256Digest pDigest2)
 {
    return (memcmp (pDigest1, pDigest2, sizeof(t_HashMD5Digest)) == 0);
 }
