@@ -9,7 +9,7 @@
 //  Module:         Information about the acquisition, creates the info file
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -29,15 +29,24 @@
 #ifndef __INFO_H__
 #define __INFO_H__
 
+#include <QObject>
+
+#ifndef __COMMON_H__
+   #include "common.h"
+#endif
+
+
 class t_InfoLocal;
 
 class t_Info: public QObject
 {
+   Q_OBJECT
+
    public:
-      t_Info (t_pcDevice pDevice=NULL);
+      t_Info (t_pcAcquisition pAcquisition=NULL);
      ~t_Info (void);
 
-      void   SetDevice  (t_pcDevice pDevice);
+//      void   SetAcquisiiton  (t_pcAcquisition pAcquisition);
       APIRET Create     (void);
       APIRET vWrite     (const char *pFormat, va_list pArguments);
       APIRET Write      (const char *pFormat, ...) __attribute__ ((format (printf, 2, 3))); // Non-static class functions have implicit this argument, thus we start counting the parameters for __attribute__ at 2, not 1 (see GNU C-Lib Function-Atttributes.html)
