@@ -10,7 +10,7 @@
 //                  widget
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -48,6 +48,7 @@
 #include "threadhash.h"
 #include "threadcompress.h"
 #include "threadwrite.h"
+#include "runstats.h"
 
 
 // -----------------------------
@@ -60,48 +61,82 @@ const int MAINWINDOW_WAIT_FOR_THREADS_TO_ABORT = 5000;
 //  Table model for device list
 // -----------------------------
 
+static t_MainWindowColumn MainWindowColumnArr[] =
+{
+   { MAINWINDOW_COLUMN_SerialNr           , t_Device::GetSerialNumber  , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_LinuxDevice        , t_Device::GetLinuxDevice   , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_Model              , t_Device::GetModel         , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_NativePath         , t_Device::GetNativePath    , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_ByPath             , t_Device::GetByPath        , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_Interface          , t_Device::GetInterface     , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_State              , t_Device::GetState         , t_ItemDelegate::DISPLAYTYPE_STATE    },
+   { MAINWINDOW_COLUMN_AdditionalStateInfo, t_Device::GetAddStateInfo  , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_Size               , t_Device::GetSizeHuman     , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_HiddenAreas        , t_Device::GetHiddenAreas   , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_BadSectors         , t_Device::GetBadSectorCount, t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_Progress           , t_Device::GetProgress      , t_ItemDelegate::DISPLAYTYPE_PROGRESS },
+   { MAINWINDOW_COLUMN_AverageSpeed       , t_Device::GetAverageSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_TimeRemaining      , t_Device::GetRemaining     , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_FifoUsage          , t_Device::GetFifoStatus    , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_SectorSizeLog      , t_Device::GetSectorSize    , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_SectorSizePhys     , t_Device::GetSectorSizePhys, t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_CurrentSpeed       , t_Device::GetCurrentSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_UserField          , t_Device::GetUserField     , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { MAINWINDOW_COLUMN_Examiner           , t_Device::GetExaminer      , t_ItemDelegate::DISPLAYTYPE_STANDARD },
+   { NULL                                 , NULL                       , 0                                    }
+};
+
+t_pMainWindowColumn MainWindowGetColumn (const char *pName)
+{
+   t_pMainWindowColumn pCol;
+
+   for (pCol=&MainWindowColumnArr[0]; pCol->pName != NULL; pCol++)
+      if (strcasecmp (pCol->pName, pName) == 0)
+         break;
+   if (pCol->pName == NULL)
+      pCol = NULL;
+
+   return pCol;
+}
+
+bool MainWindowColumnExists (const char *pName)
+{
+   return (MainWindowGetColumn (pName) != NULL);
+}
+
 t_DeviceListModel::t_DeviceListModel ()
 {
    CHK_EXIT (ERROR_MAINWINDOW_CONSTRUCTOR_NOT_SUPPORTED)
 } //lint !e1401 Not initialised
 
 t_DeviceListModel::t_DeviceListModel (t_pDeviceList pDeviceList)
-         : QAbstractTableModel()
+   : QAbstractTableModel()
 {
-   t_ColAssoc ColAssoc;
+   t_pcCfgColumn       pColCfg;
+   t_pMainWindowColumn pColDef;
+   int                  Cols;
+   t_ColAssoc           ColAssoc;
 
    poDeviceList = pDeviceList;
 
-   #define COL_ASSOC(Str, pDataFn, Display, Align, MinDx)  \
-      ColAssoc.Name        = Str;            \
-      ColAssoc.pGetDataFn  = pDataFn;        \
-      ColAssoc.DisplayType = Display;        \
-      ColAssoc.Alignment   = Align;          \
-      ColAssoc.MinWidth    = MinDx;          \
-      ColAssocList.append (ColAssoc);
-   #define LEFT   (Qt::AlignLeft    | Qt::AlignVCenter)
-   #define RIGHT  (Qt::AlignRight   | Qt::AlignVCenter)
-   #define CENTER (Qt::AlignHCenter | Qt::AlignVCenter)
+   Cols = CfgGetColumnCount ();
+   for (int i =0; i<Cols; i++)
+   {
+      pColCfg = CfgGetColumn (i);
+      if (pColCfg->ShowInMainTable)
+      {
+         pColDef = MainWindowGetColumn (pColCfg->Name);
+         if (pColDef == NULL)
+            CHK_EXIT(ERROR_MAINWINDOW_UNKNOWN_COLUMN)
 
-      COL_ASSOC (tr("Serial\nnr."            , "Column of device table"), t_Device::GetSerialNumber  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-      COL_ASSOC (tr("Linux\ndevice"          , "Column of device table"), t_Device::GetLinuxDevice   , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-      COL_ASSOC (tr("Model"                  , "Column of device table"), t_Device::GetModel         , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-      COL_ASSOC (tr("State"                  , "Column of device table"), t_Device::GetState         , t_ItemDelegate::DISPLAYTYPE_STATE   , LEFT  , 200)
-//    COL_ASSOC (tr("Additional\nstate info" , "Column of device table"), t_Device::GetAddStateInfo  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-      COL_ASSOC (tr("Size"                   , "Column of device table"), t_Device::GetSizeHuman     , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
-      COL_ASSOC (tr("Hidden\nAreas"          , "Column of device table"), t_Device::GetHiddenAreas   , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
-      COL_ASSOC (tr("Bad\nsectors"           , "Column of device table"), t_Device::GetBadSectorCount, t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
-      COL_ASSOC (tr("Progress"               , "Column of device table"), t_Device::GetProgress      , t_ItemDelegate::DISPLAYTYPE_PROGRESS, LEFT  ,   0)
-      COL_ASSOC (tr("Average\nSpeed\n[MB/s]" , "Column of device table"), t_Device::GetAverageSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
-      COL_ASSOC (tr("Time\nremaining"        , "Column of device table"), t_Device::GetRemaining     , t_ItemDelegate::DISPLAYTYPE_STANDARD, CENTER,   0)
-      COL_ASSOC (tr("FIFO queues\nusage\n[%]", "Column of device table"), t_Device::GetFifoStatus    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-//    COL_ASSOC (tr("Sector\nsize\nlog."     , "Column of device table"), t_Device::GetSectorSize    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-//    COL_ASSOC (tr("Sector\nsize\nphys."    , "Column of device table"), t_Device::GetSectorSizePhys, t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-//    COL_ASSOC (tr("Current\nSpeed\n[MB/s]" , "Column of device table"), t_Device::GetCurrentSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-   #undef COL_ASSOC
-   #undef LEFT
-   #undef RIGHT
-   #undef CENTER
+         ColAssoc.Name        = tr(pColCfg->Name);
+         ColAssoc.pGetDataFn  = pColDef->pGetDataFn;
+         ColAssoc.DisplayType = pColDef->DisplayType;
+         ColAssoc.Alignment   = pColCfg->Alignment | Qt::AlignVCenter;
+         ColAssoc.MinWidth    = pColCfg->MinWidth;
+         ColAssocList.append (ColAssoc);
+      }
+   }
 }
 
 int t_DeviceListModel::rowCount (const QModelIndex & /*parent*/) const
@@ -216,6 +251,8 @@ class t_MainWindowLocal
       QAction               *pActionAutoExit;
       t_ThreadScanWorker    *pScanWorker;
       QTimer                *pTimerRefresh;
+      QTimer                *pTimerRunStats;
+      t_pRunStats            pRunStats;
       bool                    AutoExit;
       bool                    AllAbortedByUser;
 };
@@ -226,6 +263,7 @@ static APIRET MainWindowRegisterErrorCodes (void)
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_INVALID_COLUMN))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_INVALID_DATATYPE))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_DEVICE_NOT_FOUND))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_UNKNOWN_COLUMN))
 
    return NO_ERROR;
 }
@@ -236,6 +274,8 @@ APIRET t_MainWindow::CreateMenu (void)
    QToolBar *pToolBar;
    QMenu    *pMenu;
 
+   QTUTIL_SET_FONT (pMenuBar, FONTOBJECT_MENU)
+
    // Actions
    // -------
    pOwn->pActionRescan = new QAction (tr("&Rescan"), this);
@@ -245,16 +285,19 @@ APIRET t_MainWindow::CreateMenu (void)
    // Menu
    // ----
    pMenu = pMenuBar->addMenu (tr("&Devices"));
+   QTUTIL_SET_FONT (pMenu, FONTOBJECT_MENU)
    pMenu->addAction (pOwn->pActionRescan);
    pMenu->addAction (tr("Add special device", "Menu entry"), this, SLOT(SlotAddSpecialDevice()));
 
    pMenu = pMenuBar->addMenu (tr("&Misc", "Menu entry"));
+   QTUTIL_SET_FONT (pMenu, FONTOBJECT_MENU)
    pOwn->pActionAutoExit = pMenu->addAction (tr("Exit after acquisitions completed"));
    pOwn->pActionAutoExit->setCheckable(true);
    pOwn->pActionAutoExit->setChecked ((CONFIG(AutoExit)!=0));
    pMenu->addAction (tr("Debug", "Menu entry"), this, SLOT(SlotDebug()));
 
    pMenu = pMenuBar->addMenu (tr("&Help", "Menu entry"));
+   QTUTIL_SET_FONT (pMenu, FONTOBJECT_MENU)
    pMenu->addAction (tr("About &GUYMAGER", "Menu entry"), this, SLOT(SlotAboutGuymager()));  //lint !e64 !e119 Lint reports about type mismatches and
    pMenu->addAction (tr("About &Qt"      , "Menu entry"), this, SLOT(SlotAboutQt      ()));  //lint !e64 !e119 too many arguments, Lint is probably wrong
 
@@ -263,6 +306,7 @@ APIRET t_MainWindow::CreateMenu (void)
    // -------
    pToolBar = addToolBar (QString());
    pToolBar->addAction (pOwn->pActionRescan);
+   QTUTIL_SET_FONT (pToolBar, FONTOBJECT_TOOLBAR)
 
    return NO_ERROR;
 }
@@ -288,6 +332,7 @@ t_MainWindow::t_MainWindow (t_pDeviceList pDeviceList, QWidget *pParent, Qt::WFl
    pOwn->pDeviceList = pDeviceList;
    pOwn->AutoExit         = false;
    pOwn->AllAbortedByUser = false;
+   pOwn->pRunStats        = new t_RunStats (pDeviceList);
 
    CHK_EXIT (CreateMenu ())
    setWindowTitle ("GUYMAGER");
@@ -303,7 +348,7 @@ t_MainWindow::t_MainWindow (t_pDeviceList pDeviceList, QWidget *pParent, Qt::WFl
    QVBoxLayout *pLayout = new QVBoxLayout (pOwn->pCentralWidget);
 
    pOwn->pTable                      = new t_Table (pOwn->pCentralWidget, this, pDeviceList);
-   pOwn->pDeviceListModel            = new t_DeviceListModel     (pDeviceList);
+   pOwn->pDeviceListModel            = new t_DeviceListModel (pDeviceList);
    pOwn->pProxyModel                 = new QSortFilterProxyModel (pOwn->pCentralWidget);
    pOwn->pProxyModel->setSourceModel (pOwn->pDeviceListModel);
    pOwn->pTable     ->setModel       (pOwn->pProxyModel);
@@ -324,19 +369,48 @@ t_MainWindow::t_MainWindow (t_pDeviceList pDeviceList, QWidget *pParent, Qt::WFl
 
    pOwn->pThreadScan = new t_ThreadScan ();
    CHK_EXIT (pOwn->pThreadScan->Start (&pOwn->pScanWorker))
-   CHK_QT_EXIT (connect (pOwn->pActionRescan, SIGNAL (triggered ()),          pOwn->pScanWorker, SLOT (SlotRescan ())))
+   CHK_QT_EXIT (connect (pOwn->pActionRescan, SIGNAL (triggered ()),          pOwn->pScanWorker, SLOT (SlotRescan       ())))
+   CHK_QT_EXIT (connect (pOwn->pActionRescan, SIGNAL (triggered ()),                       this, SLOT (SlotScanStarted  ())))
    CHK_QT_EXIT (connect (pOwn->pScanWorker  , SIGNAL (SignalScanFinished (t_pDeviceList)), this, SLOT (SlotScanFinished (t_pDeviceList))))
+   SlotScanStarted();
    QTimer::singleShot (300, pOwn->pScanWorker, SLOT(SlotRescan()));  // pOwn->pScanWorker->SlotRescan must not be called directly as it would be called from the
                                                                      // wrong thread! Using the signal/slot connection makes it behave correctly (see
                                                                      // Qt_AutoConnection, which is the default for connecting signals and slots).
-
    // Screen refresh timer
    // --------------------
    pOwn->pTimerRefresh = new QTimer(this);
    CHK_QT_EXIT (connect (pOwn->pTimerRefresh, SIGNAL(timeout()), this, SLOT(SlotRefresh())))
    pOwn->pTimerRefresh->start (CONFIG(ScreenRefreshInterval));
+
+   // RunStats timer
+   // --------------
+   pOwn->pTimerRunStats = new QTimer(this); // Always create the timer (in order not to check if it exists before deleting it)
+   if (pOwn->pRunStats->IsConfigured())
+   {
+      CHK_QT_EXIT (connect (pOwn->pTimerRunStats, SIGNAL(timeout()), this, SLOT(SlotUpdateRunStats())))
+      pOwn->pTimerRunStats->start (CONFIG(RunStatsInterval)*1000);
+      if (CONFIG(RunStatsInterval) > 5)                                  // Do not let the user wait too long for
+         QTimer::singleShot(3000, this, SLOT(SlotUpdateRunStats()));     // the first output file to appear
+   }
 }
 
+APIRET t_MainWindow::CheckDisconnectTimeout (void)  // Check for devices staying too long in
+{                                                   // state disconnected
+   t_pDevice pDev;
+
+   for (int i=0; i < pOwn->pDeviceList->count(); i++)
+   {
+      pDev = pOwn->pDeviceList->at(i);
+      if (pDev->DisconnectTimeoutReached())
+      {
+         LOG_INFO("[%s] Aborting acquisition because of disconnection timeout (SN: %s)", QSTR_TO_PSZ (pDev->LinuxDevice), QSTR_TO_PSZ(pDev->SerialNumber));
+         pDev->Error.Set (t_Device::t_Error::DisconnectTimeout);
+      }
+   }
+   return NO_ERROR;
+}
+
+
 void t_MainWindow::SlotRefresh (void)
 {
    t_pDevice pDevice;
@@ -344,8 +418,13 @@ void t_MainWindow::SlotRefresh (void)
    pOwn->pDeviceListModel->SlotRefresh();
    CHK_EXIT (pOwn->pTable->GetDeviceUnderCursor (pDevice))
    pOwn->pInfoField->SlotShowInfo (pDevice);
+   CHK_EXIT (CheckDisconnectTimeout())
 }
 
+void t_MainWindow::SlotUpdateRunStats (void)
+{
+   pOwn->pRunStats->Update ();
+}
 
 void t_MainWindow::SlotAddSpecialDevice (void)
 {
@@ -380,9 +459,9 @@ void t_MainWindow::SlotAddSpecialDevice (void)
          }
 
          if (Error)
-            QMessageBox::information (this, tr ("Invalid device", "Dialog title"), tr("The device or file cannot be selected because its size is unknown."), QMessageBox::Ok);
+            t_MessageBox::information (this, tr ("Invalid device", "Dialog title"), tr("The device or file cannot be selected because its size is unknown."), QMessageBox::Ok);
          else if (Size <= 0)
-            QMessageBox::information (this, tr ("Invalid device", "Dialog title"), tr("The device or file cannot be selected because it contains 0 bytes."), QMessageBox::Ok);
+            t_MessageBox::information (this, tr ("Invalid device", "Dialog title"), tr("The device or file cannot be selected because it contains 0 bytes."), QMessageBox::Ok);
          if (Size <= 0)
             continue;
       }
@@ -392,7 +471,7 @@ void t_MainWindow::SlotAddSpecialDevice (void)
       if (pDeviceFound)
       {
          delete pDevice;
-         QMessageBox::information (this, tr ("Device alrady contained", "Dialog title"), tr("The selected file or device already is contained in the table."), QMessageBox::Ok);
+         t_MessageBox::information (this, tr ("Device alrady contained", "Dialog title"), tr("The selected file or device already is contained in the table."), QMessageBox::Ok);
          continue;
       }
       break;
@@ -419,6 +498,11 @@ APIRET t_MainWindow::RemoveSpecialDevice (t_pDevice pDevice)
    return NO_ERROR;
 }
 
+void t_MainWindow::SlotScanStarted (void)
+{
+   pOwn->pActionRescan->setEnabled(false);
+}
+
 
 // SlotScanFinished provides us with the new device list. Now, our current device list needs to be updated. The rules are:
 //  (1) For devices, who are in both lists:
@@ -457,7 +541,7 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
       CHK_EXIT (pDeviceList->MatchDevice (pNewDev, pDev))
 //      if (pDev)
 //      {
-//         LOG_INFO ("device %s %s %s %llu matches %s %s %s %llu", 
+//         LOG_INFO ("device %s %s %s %llu matches %s %s %s %llu",
 //            QSTR_TO_PSZ (pNewDev->LinuxDevice), QSTR_TO_PSZ (pNewDev->Model), QSTR_TO_PSZ (pNewDev->SerialNumber), pNewDev->Size,
 //            QSTR_TO_PSZ (   pDev->LinuxDevice), QSTR_TO_PSZ (   pDev->Model), QSTR_TO_PSZ (   pDev->SerialNumber), pNewDev->Size)
 //      }
@@ -474,14 +558,17 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
       if (!NewDevice)
       {
          pDev->Checked = true;
-         if ((pDev->State == t_Device::AcquirePaused) ||
-             (pDev->State == t_Device::VerifyPaused))
+         pDev->NativePath  = pNewDev->NativePath;     // The device might have been connected to a
+         pDev->ByPath      = pNewDev->ByPath;         // different port.
+         pDev->Interface   = pNewDev->Interface;
+         if ((pDev->GetState() == t_Device::AcquirePaused) ||
+             (pDev->GetState() == t_Device::VerifyPaused))
          {
             LOG_INFO ("Switching %s to connected again", QSTR_TO_PSZ(pNewDev->LinuxDevice))
-               pDev->LinuxDevice = pNewDev->LinuxDevice;    // Linux may choose a new path when reconnecting the device
-            if (pDev->State == t_Device::AcquirePaused)  // (1)
-                 pDev->State = t_Device::Acquire;
-            else pDev->State = t_Device::Verify;
+            pDev->LinuxDevice = pNewDev->LinuxDevice;    // Linux may choose a new path when reconnecting the device
+            if (pDev->GetState() == t_Device::AcquirePaused)  // (1)
+                 pDev->SetState (t_Device::Acquire);
+            else pDev->SetState (t_Device::Verify );
          }
       }
       else
@@ -498,13 +585,13 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
       pDev = pDeviceList->at (i);
       if (!pDev->Checked)
       {
-         switch (pDev->State)
+         switch (pDev->GetState())
          {
-            case t_Device::Acquire      : pDev->State = t_Device::AcquirePaused; break;  // (2b)
-            case t_Device::Verify       : pDev->State = t_Device::VerifyPaused ; break;  // (2b)
+            case t_Device::Acquire      : pDev->SetState (t_Device::AcquirePaused); break;  // (2b)
+            case t_Device::Verify       : pDev->SetState (t_Device::VerifyPaused ); break;  // (2b)
             case t_Device::AcquirePaused: break;
             case t_Device::VerifyPaused : break;
-            default                     : pDeviceList->removeAt (i--);                   // (2a)
+            default                     : pDeviceList->removeAt (i--);                     // (2a)
                                           delete pDev;
          }
       }
@@ -514,6 +601,7 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
 
    pOwn->pDeviceListModel->SlotRefresh();
    pOwn->pTable->resizeColumnsToContents();
+   pOwn->pActionRescan->setEnabled(true);
 }
 
 void t_MainWindow::closeEvent (QCloseEvent *pEvent)
@@ -527,18 +615,18 @@ void t_MainWindow::closeEvent (QCloseEvent *pEvent)
    for (i=0; (i<pOwn->pDeviceList->count()) && (!AcquisitionsActive); i++)
    {
       pDevice = pOwn->pDeviceList->at(i);
-      AcquisitionsActive = (pDevice->State == t_Device::Acquire)       ||
-                           (pDevice->State == t_Device::Verify )       ||
-                           (pDevice->State == t_Device::Cleanup)       ||
-                           (pDevice->State == t_Device::AcquirePaused) ||
-                           (pDevice->State == t_Device::VerifyPaused );
+      AcquisitionsActive = (pDevice->GetState() == t_Device::Acquire)       ||
+                           (pDevice->GetState() == t_Device::Verify )       ||
+                           (pDevice->GetState() == t_Device::Cleanup)       ||
+                           (pDevice->GetState() == t_Device::AcquirePaused) ||
+                           (pDevice->GetState() == t_Device::VerifyPaused );
    }
 
    if (AcquisitionsActive)
    {
       LOG_INFO ("Some acquisitions still active - ask user if he really wants to quit")
-      Button = QMessageBox::question (this, tr ("Exit GUYMAGER", "Dialog title"), tr("There are active acquisitions. Do you really want to abort them and quit?"),
-                                      QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
+      Button = t_MessageBox::question (this, tr ("Exit GUYMAGER", "Dialog title"), tr("There are active acquisitions. Do you really want to abort them and quit?"),
+                                       QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
       if (Button == QMessageBox::Yes)
       {  // stop acquisitions
          pOwn->AllAbortedByUser = true;
@@ -585,7 +673,9 @@ void t_MainWindow::SlotDebug (void)
       ADD_MESSAGE (pThreadHash, "Hash")
       for (int j=0; j<pDevice->ThreadCompressList.count(); j++)
          ADD_MESSAGE (ThreadCompressList.at(j), QString("Compress %1").arg(j))
-      ADD_MESSAGE (pThreadWrite, "Write")
+      ADD_MESSAGE (pThreadWrite1, "Write1")
+      if (pDevice->Duplicate)
+          ADD_MESSAGE (pThreadWrite2, "Write2")
    }
    DebugInfo += QString ("\nThreadScan: %1") .arg(pOwn->pThreadScan->GetDebugMessage());
 
@@ -642,7 +732,7 @@ void t_MainWindow::SlotAutoExit (void)
          }
          else
          {
-            LOG_INFO ("Autoexit coutdown has been stopped. Guymager will continue to run.")
+            LOG_INFO ("Autoexit countdown has been stopped. Guymager will continue to run.")
          }
       }
       else
@@ -654,12 +744,15 @@ void t_MainWindow::SlotAutoExit (void)
 
 bool t_MainWindow::AutoExit (void)
 {
-   return pOwn->AutoExit; 
+   return pOwn->AutoExit;
 }
 
 t_MainWindow::~t_MainWindow ()
 {
    CHK_EXIT (pOwn->pThreadScan->Stop ())
+   delete pOwn->pTimerRefresh;
+   delete pOwn->pTimerRunStats;
+   delete pOwn->pRunStats;
    delete pOwn->pTable;
    delete pOwn->pProxyModel;
    delete pOwn->pDeviceListModel;
