@@ -9,7 +9,7 @@
 //  Module:         MD5 calculation
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -67,7 +67,7 @@ void MD5Init (t_MD5Context *pContext)
    pContext->buflen = 0;
 }
 
-static inline void set_uint32 (char *cp, uint32_t v)
+static void set_uint32 (char *cp, uint32_t v)
 {
    memcpy (cp, &v, sizeof v);
 }
