@@ -6,10 +6,10 @@
 //                  Service de Police Judiciaire
 //                  Section Nouvelles Technologies
 // ****************************************************************************
-//  Module:         MD5 calculation
+//  Module:         SHA256 calculation
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011, 2012 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -595,7 +595,7 @@ void SHA256Init (t_pSHA256Context pContext)
    pContext->buflen   = 0;
 }
 
-static inline void set_uint32 (char *cp, uint32 v)
+static void set_uint32 (char *cp, uint32 v)
 {
    memcpy (cp, &v, sizeof v);
 }
