@@ -32,7 +32,6 @@
 #include <unistd.h>
 #include <QtGui>
 
-#include "toolconstants.h"
 #include "common.h"
 #include "config.h"
 #include "threadread.h"
@@ -48,8 +47,12 @@
 #include "dlgabort.h"
 #include "file.h"
 #include "qtutil.h"
+#include "util.h"
 #include "infofield.h"
 
+#include "toolconstants.h"
+#include "toolsysinfo.h"
+
 
 class t_TableLocal
 {
@@ -265,12 +268,24 @@ APIRET t_Table::InfoAcquisitionStart (t_pAcquisition pAcquisition)
    QString      FormatDescription;
    QString      FormatExtension;
    QString      Str;
+   QString      Uname;
+   t_UtilMem    Hostname(200);
+   t_UtilMem    Domainname(200);
+   char       *pHostname;
+   char       *pDomainname;
 
    CHK (pAcquisition->Info.Create())
    CHK (pAcquisition->Info.Title   (tr("GUYMAGER ACQUISITION INFO FILE", "Info file")))
 
    t_Log::GetLibGuyToolsVersion (&pLibGuyToolsVersionInstalled);
    pLibEwfVersionInstalled = (const char *) libewf_get_version ();
+   CHK (ToolSysInfoUname (Uname))
+   pHostname   = (char *) Hostname.GetpMem();
+   pDomainname = (char *) Domainname.GetpMem();
+   if (gethostname (pHostname, Hostname.Size()) != 0)
+      Hostname.Set  ("-- not available --");
+   if (getdomainname (pDomainname, Domainname.Size()) != 0)
+      Domainname.Set("-- not available --");
 
    CHK (pAcquisition->Info.WriteLn ())
    CHK (pAcquisition->Info.Title   (tr("Guymager", "Info file")))
@@ -280,6 +295,9 @@ APIRET t_Table::InfoAcquisitionStart (t_pAcquisition pAcquisition)
    CHK (pAcquisition->Info.AddRow (tr("Compiled with:: %1 %2"     , "Info file") .arg("gcc") .arg(__VERSION__)))
    CHK (pAcquisition->Info.AddRow (tr("libewf version:: %1"       , "Info file") .arg(pLibEwfVersionInstalled     )))
    CHK (pAcquisition->Info.AddRow (tr("libguytools version:: %1"  , "Info file") .arg(pLibGuyToolsVersionInstalled)))
+   CHK (pAcquisition->Info.AddRow (tr("Host name:: %1"            , "Info file") .arg(pHostname)))
+   CHK (pAcquisition->Info.AddRow (tr("Domain name:: %1"          , "Info file") .arg(pDomainname)))
+   CHK (pAcquisition->Info.AddRow (tr("System:: %1"               , "Info file") .arg(Uname)))
    CHK (pAcquisition->Info.WriteTable ());
 
    CHK (pAcquisition->Info.WriteLn ())
@@ -303,7 +321,7 @@ APIRET t_Table::InfoAcquisitionStart (t_pAcquisition pAcquisition)
    if (!FormatExtension.isEmpty())
       Str += tr(" - file extension is %1", "Info file") .arg(FormatExtension);
    CHK (pAcquisition->Info.AddRow (Str))
-   if (pAcquisition->Format != t_File::DD)
+   if ((pAcquisition->Format != t_File::DD) || CONFIG (InfoFieldsForDd))
    {
       CHK (pAcquisition->Info.AddRow (tr("Image meta data", "Info file")))
       CHK (pAcquisition->Info.AddRow ("   " + t_DlgAcquire::tr("EwfCaseNumber"    ) + QString(":: %1") .arg(pAcquisition->CaseNumber    )))
@@ -850,11 +868,17 @@ APIRET t_Table::InfoAcquisitionEnd0 (t_pAcquisition pAcquisition)
                   if (pAcquisition->CalcSHA256) CHK (HashSHA256DigestStr (&pDevice->SHA256Digest, SHA256)) // we can calculate the acquisition hash.
                }
                break;
+            case t_Device::t_Error::AcquisitionStartFailed:
+               StateStr = tr("Aborted because acquisition failed to start", "Info file");
+               break;
             case t_Device::t_Error::ReadError:
                StateStr = tr("Aborted because of source read error", "Info file");
                break;
-            case t_Device::t_Error::AcquisitionStartFailed:
-               StateStr = tr("Aborted because acquisition failed to start", "Info file");
+            case t_Device::t_Error::TooManyBadSectors:
+               StateStr = tr("Aborted, too many bad sectors (> %1)", "Info file") .arg(CONFIG(JobMaxBadSectors));
+               break;
+            case t_Device::t_Error::DisconnectTimeout:
+               StateStr = tr("Aborted because device has is inaccessible for too long (> %1s)", "Info file") .arg (CONFIG(JobDisconnectTimeout));
                break;
             default:
                switch (pDevice->Error.Get (pAcquisition->Instance))
