@@ -9,7 +9,7 @@
 //  Module:         Multi-threaded compression
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -28,8 +28,6 @@
 
 #include <QtCore>
 
-#include "libewf.h"
-
 #include "common.h"
 #include "device.h"
 #include "threadcompress.h"
@@ -85,7 +83,7 @@ APIRET t_ThreadCompress::GetFifoExtraSpace (t_pDevice pDevice, unsigned int *pEx
 
    ExtraZ = UtilGetMaxZcompressedBufferSize (pDevice->FifoBlockSize) - pDevice->FifoBlockSize;
 
-   switch (pDevice->Acquisition.Format)
+   switch (pDevice->Acquisition1.Format)
    {
       case t_File::DD  : ExtraSpace = 0;   break;
       case t_File::AAFF: ExtraSpace = AaffPreprocessExtraSpace(pDevice->FifoBlockSize);  break;
@@ -113,10 +111,7 @@ void t_ThreadCompress::run (void)
    const int       SubFifoNr = pOwn->ThreadNr;
    size_t          CompressedMaxSize = 0;
    size_t          CompressedSize    = 0;
-   void          *pHandle;
    LIBEWF_HANDLE *pEwfHandle  = NULL;
-   t_Aewf        *pAewfHandle = NULL;
-// t_Aaff        *pAaffHandle = NULL;
    ssize_t         rc;
 
    LOG_INFO ("[%s] Compression thread #%d started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), pOwn->ThreadNr)
@@ -124,25 +119,21 @@ void t_ThreadCompress::run (void)
 
    pDevice  = pOwn->pDevice;
 
-   if (pDevice->Acquisition.Format == t_File::EWF)
-      CompressedMaxSize = (size_t) (pDevice->FifoBlockSize * 1.001) + 12; // see zlib documentation
-
-   CHK_EXIT (pDevice->pThreadWrite->GetpFileHandle (&pHandle))
-   switch (pDevice->Acquisition.Format)
+   if (pDevice->Acquisition1.Format == t_File::EWF)
    {
-      case t_File::EWF:  if (CONFIG(EwfFormat) == t_File::AEWF)
-                              pAewfHandle = (t_Aewf *)        pHandle;
-                         else pEwfHandle  = (LIBEWF_HANDLE *) pHandle;
-                         break;
-      case t_File::AAFF: // pAawfHandle = (t_Aawf *) pHandle;
-                         break;
-      default:           CHK_EXIT (ERROR_THREADCOMPRESS_INVALID_FORMAT)
+      CompressedMaxSize = (size_t) (pDevice->FifoBlockSize * 1.001) + 12; // see zlib documentation
+      if (CONFIG(EwfFormat) != t_File::AEWF)
+      {
+         void *pHandle;
+         CHK_EXIT (pDevice->pThreadWrite1->GetpFileHandle (&pHandle))  // This handle is the reason why libewf can't
+         pEwfHandle  = (LIBEWF_HANDLE *) pHandle;                      // be used together with image duplication
+      }
    }
 
    CHK_EXIT (pDevice->pFifoCompressIn ->GetSubFifo (SubFifoNr, pFifoIn ))
    CHK_EXIT (pDevice->pFifoCompressOut->GetSubFifo (SubFifoNr, pFifoOut))
 
-   while (!Finished && !pDevice->AbortRequest)
+   while (!Finished && !pDevice->Error.Abort())
    {
       CHK_EXIT (SetDebugMessage ("Wait for block"))
       CHK_EXIT (pFifoIn->Get (pFifoBlockIn))
@@ -154,20 +145,20 @@ void t_ThreadCompress::run (void)
          pFifoBlockOut->Nr       = pFifoBlockIn->Nr;
          pFifoBlockOut->DataSize = pFifoBlockIn->DataSize;
 
-         switch (pDevice->Acquisition.Format)
+         switch (pDevice->Acquisition1.Format)
          {
             case t_File::EWF:
                if (CONFIG(EwfFormat) == t_File::AEWF)
                {
-                  CHK_EXIT (AewfPreprocess (pAewfHandle, &pFifoBlockOut->pAewfPreprocess, pFifoBlockIn ->Buffer, pFifoBlockIn ->DataSize,
-                                                                                          pFifoBlockOut->Buffer, pFifoBlockOut->BufferSize))
-                  if (pFifoBlockOut->pAewfPreprocess->Compressed)
+                  CHK_EXIT (AewfPreprocess (&pFifoBlockOut->AewfPreprocess, pFifoBlockIn ->Buffer, pFifoBlockIn ->DataSize,
+                                                                            pFifoBlockOut->Buffer, pFifoBlockOut->BufferSize))
+                  if (pFifoBlockOut->AewfPreprocess.Compressed)
                   {
                      CHK_EXIT (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlockIn))
                   }
                   else
                   {
-                     pFifoBlockIn->pAewfPreprocess = pFifoBlockOut->pAewfPreprocess;
+                     pFifoBlockIn->AewfPreprocess = pFifoBlockOut->AewfPreprocess;
                      CHK_EXIT (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlockOut))
                      pFifoBlockOut = pFifoBlockIn;
                   }
@@ -175,12 +166,20 @@ void t_ThreadCompress::run (void)
                }
                else
                {
-                  CompressedSize = CompressedMaxSize;   // Must be initialised with the max buffer size (we use this one instead of MULTITHREADED_COMPRESSION_FIFO_BLOCK_SIZE in order to check if ti works as tols in the zlib docu)
-                  rc = libewf_raw_write_prepare_buffer (pEwfHandle, pFifoBlockIn ->Buffer, pFifoBlockIn->DataSize,
-                                                                    pFifoBlockOut->Buffer, &CompressedSize,
-                                                                   &pFifoBlockOut->EwfCompressionUsed,
-                                                                   &pFifoBlockOut->EwfChunkCRC,
-                                                                   &pFifoBlockOut->EwfWriteCRC);
+                  CompressedSize = CompressedMaxSize;  // Must be initialised with the max buffer size (we use this one instead of MULTITHREADED_COMPRESSION_FIFO_BLOCK_SIZE in order to check if it works as written in the zlib docu)
+                  #if (LIBEWF_VERSION < 20130416)
+                     rc = libewf_raw_write_prepare_buffer (pEwfHandle, pFifoBlockIn ->Buffer, pFifoBlockIn->DataSize,
+                                                                       pFifoBlockOut->Buffer, &CompressedSize,
+                                                                      &pFifoBlockOut->EwfCompressionUsed,
+                                                                      &pFifoBlockOut->EwfChunkCRC,
+                                                                      &pFifoBlockOut->EwfWriteCRC);
+                  #else
+                     rc = libewf_handle_prepare_write_chunk(pEwfHandle, pFifoBlockIn ->Buffer, pFifoBlockIn->DataSize,
+                                                                        pFifoBlockOut->Buffer, &CompressedSize,
+                                                                       &pFifoBlockOut->EwfCompressionUsed,
+                                                                       &pFifoBlockOut->EwfChunkCRC,
+                                                                       &pFifoBlockOut->EwfWriteCRC, NULL);
+                  #endif
                   if (pFifoBlockOut->EwfCompressionUsed)
                   {
                      pFifoBlockOut->EwfDataSize = CompressedSize;                        // Data to be forwarded is contained in
@@ -208,14 +207,14 @@ void t_ThreadCompress::run (void)
                break;
 
             case t_File::AAFF:
-               CHK_EXIT (AaffPreprocess (&pFifoBlockOut->pAaffPreprocess, pFifoBlockIn->Buffer, pFifoBlockIn->DataSize, pFifoBlockOut->Buffer, pFifoBlockOut->BufferSize))
-               if (pFifoBlockOut->pAaffPreprocess->Compressed)
+               CHK_EXIT (AaffPreprocess (&pFifoBlockOut->AaffPreprocess, pFifoBlockIn->Buffer, pFifoBlockIn->DataSize, pFifoBlockOut->Buffer, pFifoBlockOut->BufferSize))
+               if (pFifoBlockOut->AaffPreprocess.Compressed)
                {
                   CHK_EXIT (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlockIn))
                }
                else
                {
-                  pFifoBlockIn->pAaffPreprocess = pFifoBlockOut->pAaffPreprocess;
+                  pFifoBlockIn->AaffPreprocess = pFifoBlockOut->AaffPreprocess;
                   CHK_EXIT (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlockOut))
                   pFifoBlockOut = pFifoBlockIn;
                }
@@ -232,7 +231,8 @@ void t_ThreadCompress::run (void)
          Finished = true;
       }
    }
-   CHK_EXIT (pDevice->pThreadWrite->ReleaseFileHandle ())
+   if ((pDevice->Acquisition1.Format == t_File::EWF) && (CONFIG(EwfFormat) != t_File::AEWF))
+      CHK_EXIT (pDevice->pThreadWrite1->ReleaseFileHandle ())
 
    LOG_INFO ("[%s] Compression thread #%d exits now - %Ld blocks processed", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice), pOwn->ThreadNr, Blocks)
    CHK_EXIT (SetDebugMessage ("Exit run function"))
