@@ -9,7 +9,7 @@
 //  Module:         Thread for reading data.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -27,6 +27,7 @@
 // along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
 #include <errno.h>
+#include <unistd.h>
 
 #include <QtCore>
 
@@ -85,10 +86,10 @@ t_ThreadRead::~t_ThreadRead (void)
    delete pOwn;
 }
 
-inline bool ThreadReadEOF (t_pcDevice pDevice)
+inline bool ThreadReadEOF (t_pDevice pDevice)
 {
 //   LOG_INFO ("EOF check %Ld %Ld", pDevice->CurrentPos, pDevice->Size)
-   return (pDevice->CurrentReadPos >= pDevice->Size);
+   return (pDevice->CurrentReadPos.Get() >= pDevice->Size);
 }
 
 static APIRET ThreadReadCheckDeviceExists (t_pcDevice pDevice, bool &DeviceExists)
@@ -110,9 +111,9 @@ static APIRET ThreadReadDeviceDisconnected (t_pDevice pDevice, const char* pStrA
       (void) close (pDevice->FileDescSrc);
       pDevice->FileDescSrc = t_Device::FileDescEmpty;
    }
-   if (pDevice->State == t_Device::Acquire)
-        pDevice->State = t_Device::AcquirePaused;
-   else pDevice->State = t_Device::VerifyPaused;
+   if (pDevice->GetState() == t_Device::Acquire)
+        pDevice->SetState (t_Device::AcquirePaused);
+   else pDevice->SetState (t_Device::VerifyPaused );
    LOG_INFO ("[%s] Device disconnected (%s, %d, %s), switching device state to %s", QSTR_TO_PSZ(pDevice->LinuxDevice), pStrAction, errno, strerror(errno), pDevice->StateStr())
 
    return ERROR_THREADREAD_DEVICE_DISCONNECTED;
@@ -128,7 +129,7 @@ static APIRET ThreadReadAdjustSeek (t_pDevice pDevice)        // According to th
    if (ThreadReadEOF (pDevice))
       return NO_ERROR;
 
-   rc = lseek64 (pDevice->FileDescSrc, (off64_t)pDevice->CurrentReadPos, SEEK_SET);
+   rc = lseek64 (pDevice->FileDescSrc, (off64_t)pDevice->CurrentReadPos.Get(), SEEK_SET);
    if (rc < 0)
       CHK_RET (ThreadReadDeviceDisconnected (pDevice, "seek"))
 
@@ -146,18 +147,27 @@ static APIRET ThreadReadBlock0 (t_pDevice pDevice, unsigned char *pBuffer, unsig
    if (pDevice->FileDescSrc == t_Device::FileDescEmpty)
    {
       OpenFlags = O_RDONLY | O_NOATIME;
-      if (CONFIG (FifoMemoryManager) && CONFIG (DirectIO))  // We need the buffers to be located on page size (4K) boundaries; this only
-      {                                                     // is guaranteed when working with Guymager's internal memory manager (see t_FifoMemory::t_FifoMemory)
+      if ((CONFIG (DirectIO) || pDevice->FallbackMode) && // Try DirectIO if configured to do so; Also use DirectIO in FallBack mode, in order to avoid the "contagious
+           CONFIG (FifoMemoryManager))                    // error". For DirectIO, FifoMemoryManager must be on, as the buffers must be located on page size (4K) boundaries
+      {                                                   // (this only is guaranteed when working with Guymager's internal memory manager (see t_FifoMemory::t_FifoMemory))
          LOG_INFO ("[%s] Trying direct mode", QSTR_TO_PSZ(pDevice->LinuxDevice))
          OpenFlags |= O_DIRECT;
       }
+
       pDevice->FileDescSrc = open64 (QSTR_TO_PSZ(pDevice->LinuxDevice), OpenFlags);
       if ((pDevice->FileDescSrc < 0) && (OpenFlags & O_DIRECT) && (errno == EINVAL))   // If an invalid parameter is reported, O_DIRECT might have
       {                                                                                // caused problems. This was observed when opening files
          OpenFlags &= ~O_DIRECT;                                                       // (added as special device) on the cases server (dd reported
          pDevice->FileDescSrc = open64 (QSTR_TO_PSZ(pDevice->LinuxDevice), OpenFlags); // the same error with iflag=direct). Try again without it.
       }
-      LOG_INFO ("[%s] Direct mode switched %s", QSTR_TO_PSZ(pDevice->LinuxDevice), (OpenFlags & O_DIRECT) ? "on":"off")
+      if ((pDevice->FileDescSrc < 0) && (OpenFlags & O_NOATIME))                       // Another cause for trouble might be NOATIME. This was observed when
+      {                                                                                // giving standard users read access to physical disks).
+         OpenFlags &= ~O_NOATIME;
+         pDevice->FileDescSrc = open64 (QSTR_TO_PSZ(pDevice->LinuxDevice), OpenFlags);
+      }
+      LOG_INFO ("[%s] Flag DIRECT  on source device switched %s", QSTR_TO_PSZ(pDevice->LinuxDevice), (OpenFlags & O_DIRECT ) ? "on":"off")
+      LOG_INFO ("[%s] Flag NOATIME on source device switched %s", QSTR_TO_PSZ(pDevice->LinuxDevice), (OpenFlags & O_NOATIME) ? "on":"off")
+
       if (pDevice->FileDescSrc < 0)
          CHK_RET (ThreadReadDeviceDisconnected (pDevice, "open"))
       CHK_RET (ThreadReadAdjustSeek (pDevice))
@@ -166,14 +176,14 @@ static APIRET ThreadReadBlock0 (t_pDevice pDevice, unsigned char *pBuffer, unsig
    Read = read (pDevice->FileDescSrc, pBuffer, Size);
 
 // Bad sector simulation
-//   if ((pDevice->CurrentReadPos > 500000) && (pDevice->CurrentReadPos < 600000) ||
-//       (pDevice->CurrentReadPos > 700000) && (pDevice->CurrentReadPos < 740000) ||
-//       (pDevice->CurrentReadPos == 0) || (pDevice->CurrentReadPos == 512) || (pDevice->CurrentReadPos == 1024))
-//      Read = 0;
-
+// if ((pDevice->CurrentReadPos.Get() > 900000000) && (pDevice->CurrentReadPos.Get() < (900000000+65536*2)))
+// {
+//    #warning "bad sector simulation is still on"
+//    Read=0;
+// }
    if (Read == (ssize_t)Size)
    {
-      pDevice->CurrentReadPos += Size;
+      pDevice->CurrentReadPos.Inc(Size);
       return NO_ERROR;
    }
    else
@@ -208,8 +218,8 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
    APIRET          RcSeek;
 
    pDevice       = pOwn->pDevice;
-   PrevReadPos   = pDevice->CurrentReadPos;
-   RemainingSize = pDevice->Size - pDevice->CurrentReadPos;
+   PrevReadPos   = pDevice->CurrentReadPos.Get();
+   RemainingSize = pDevice->Size - PrevReadPos;
    if ((quint64)pDevice->FifoBlockSize > RemainingSize)
         BytesToRead = RemainingSize;
    else BytesToRead = pDevice->FifoBlockSize;
@@ -239,25 +249,35 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
             if (!pDevice->FallbackMode)
             {
                pDevice->FallbackMode = true;
-//               ReadTry = (int) GETMIN (pDevice->SectorSize, pDevice->SectorSizePhys);
+               if (!CONFIG(DirectIO))                             // When running without DirectIO we close the source device,
+               {                                                  // thus forcing ThreadReadBlock0 to re-open it with DirectIO
+                  (void) close (pDevice->FileDescSrc);            // switched on (in order to avoid the "contagious error")
+                  pDevice->FileDescSrc = t_Device::FileDescEmpty;
+               }
                ReadTry = pDevice->SectorSize;
                LOG_INFO ("[%s] Device read error, switching to slow fallback mode. Reading sectors individually from now on.", QSTR_TO_PSZ(pDevice->LinuxDevice))
             }
             else
             {
-               Sector = pDevice->CurrentReadPos / pDevice->SectorSize;
-               pDevice->CurrentReadPos += ReadTry;
+               Sector = pDevice->CurrentReadPos.Get() / pDevice->SectorSize;
+               pDevice->CurrentReadPos.Inc(ReadTry);
                RcSeek = ThreadReadAdjustSeek (pDevice); // If the seek still succeeds, then there's probably just a bad sector on
                switch (RcSeek)                          // the source device, let's replace it by a zero sector.
                {
                   case NO_ERROR:
-                     if ((pDevice->Acquisition.Format == t_File::AAFF) && (CONFIG (AffMarkBadSectors)))
+                     if ((pDevice->Acquisition1.Format == t_File::AAFF) && (CONFIG (AffMarkBadSectors)))
                           CHK (AaffCopyBadSectorMarker (pBuffer, ReadTry))
                      else memset (pBuffer, 0, ReadTry);
                      BytesRead += ReadTry;
                      pBuffer   += ReadTry;
                      LOG_INFO ("[%s] Read error on sector %Ld, bad data replaced by zero block", QSTR_TO_PSZ(pDevice->LinuxDevice), Sector)
                      pDevice->AddBadSector (Sector);
+
+                     if (CONFIG(JobMaxBadSectors) &&
+                        (CONFIG(LimitJobs) != CONFIG_LIMITJOBS_OFF) &&
+                        (pDevice->GetBadSectorCount(false) > (quint64)CONFIG(JobMaxBadSectors)))
+                        pDevice->Error.Set (t_Device::t_Error::TooManyBadSectors);
+
                      break;
                   case ERROR_THREADREAD_DEVICE_DISCONNECTED:
                      break;
@@ -273,19 +293,19 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
          default:
             CHK (RcRead)
       }
-   } while ((BytesRead < BytesToRead) && (pDevice->State != t_Device::AcquirePaused)
-                                      && (pDevice->State != t_Device::VerifyPaused )
-                                      && !pDevice->AbortRequest);
+   } while ((BytesRead < BytesToRead) && (pDevice->GetState() != t_Device::AcquirePaused)
+                                      && (pDevice->GetState() != t_Device::VerifyPaused )
+                                      && !pDevice->Error.Abort());
 
-   if ((pDevice->State == t_Device::AcquirePaused) ||
-       (pDevice->State == t_Device::VerifyPaused ))
+   if ((pDevice->GetState() == t_Device::AcquirePaused) ||
+       (pDevice->GetState() == t_Device::VerifyPaused ))
    {
       CHK (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlock))
-      pDevice->CurrentReadPos = PrevReadPos;
+      pDevice->CurrentReadPos.Set(PrevReadPos);
 
       return ERROR_THREADREAD_DEVICE_DISCONNECTED;
    }
-   else if (pDevice->AbortRequest)
+   else if (pDevice->Error.Abort())
    {
       CHK (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlock))
 
@@ -295,6 +315,11 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
    {
       pDevice->FallbackMode = false;  // Switch to fast reading mode if last read was good
       LOG_INFO ("[%s] Device read ok again, switching to fast block read.", QSTR_TO_PSZ(pDevice->LinuxDevice))
+      if (!CONFIG(DirectIO))                                  // Be sure to switch DirectIO off again if configured so.
+      {                                                       // We do so by closing the source device. ThreadReadBlock0
+         (void) close (pDevice->FileDescSrc);                 // is then forced to re-open the device where it will switch
+         pDevice->FileDescSrc = t_Device::FileDescEmpty;      // off DirectIO.
+      }
    }
 
    pFifoBlock->LastBlock = ThreadReadEOF (pDevice);
@@ -309,6 +334,7 @@ void t_ThreadRead::run (void)
    APIRET              rc;
    int                 rcf;
    bool                CalcHashMD5;
+   bool                CalcHashSHA1;
    bool                CalcHashSHA256;
    quint64           *pBlocks;
    quint64             BlocksRead     = 0;
@@ -316,13 +342,15 @@ void t_ThreadRead::run (void)
    quint64             BytesRead      = 0;
    quint64             BytesVerified  = 0;
    t_HashContextMD5    HashContextMD5;
+   t_HashContextSHA1   HashContextSHA1;
    t_HashContextSHA256 HashContextSHA256;
 
    LOG_INFO ("[%s] Read thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
    CHK_EXIT (SetDebugMessage ("Start run function"))
    pDevice = pOwn->pDevice;
-   CalcHashMD5    = pDevice->Acquisition.CalcMD5    && !CONFIG (UseSeparateHashThread);
-   CalcHashSHA256 = pDevice->Acquisition.CalcSHA256 && !CONFIG (UseSeparateHashThread);
+   CalcHashMD5    = pDevice->Acquisition1.CalcMD5    && !CONFIG (UseSeparateHashThread);
+   CalcHashSHA1   = pDevice->Acquisition1.CalcSHA1   && !CONFIG (UseSeparateHashThread);
+   CalcHashSHA256 = pDevice->Acquisition1.CalcSHA256 && !CONFIG (UseSeparateHashThread);
 
    pBlocks = &BlocksRead;
 
@@ -331,14 +359,15 @@ void t_ThreadRead::run (void)
 //   pDevice->AddBadSector (5);
 //   pDevice->AddBadSector (122);
 
-   pDevice->State = t_Device::Acquire;
+   pDevice->SetState (t_Device::Acquire);
    for (;;)
    {
-      pDevice->FileDescSrc    = t_Device::FileDescEmpty;
-      pDevice->CurrentReadPos = 0;
-      pDevice->FallbackMode   = false;
+      pDevice->CurrentReadPos.Zero();
+      pDevice->FileDescSrc   = t_Device::FileDescEmpty;
+      pDevice->FallbackMode  = false;
       *pBlocks = 0;
       if (CalcHashMD5)    CHK_EXIT (HashMD5Init    (&HashContextMD5   ))
+      if (CalcHashSHA1)   CHK_EXIT (HashSHA1Init   (&HashContextSHA1  ))
       if (CalcHashSHA256) CHK_EXIT (HashSHA256Init (&HashContextSHA256))
       do
       {
@@ -354,11 +383,12 @@ void t_ThreadRead::run (void)
                pFifoBlock->Nr = (*pBlocks);
 
                if (CalcHashMD5)    CHK_EXIT (HashMD5Append    (&HashContextMD5   , pFifoBlock->Buffer, pFifoBlock->DataSize))
+               if (CalcHashSHA1)   CHK_EXIT (HashSHA1Append   (&HashContextSHA1  , pFifoBlock->Buffer, pFifoBlock->DataSize))
                if (CalcHashSHA256) CHK_EXIT (HashSHA256Append (&HashContextSHA256, pFifoBlock->Buffer, pFifoBlock->DataSize))
 
-               if ((pDevice->State == t_Device::Verify) && (CalcHashMD5 || CalcHashSHA256))
+               if ((pDevice->GetState() == t_Device::Verify) && (CalcHashMD5 || CalcHashSHA1 || CalcHashSHA256))
                {
-                  pDevice->IncCurrentVerifyPosSrc (pFifoBlock->DataSize);
+                  pDevice->CurrentVerifyPosSrc.Inc (pFifoBlock->DataSize);
                   CHK_EXIT (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlock))
                }
                else
@@ -369,8 +399,8 @@ void t_ThreadRead::run (void)
                break;
 
             case ERROR_THREADREAD_DEVICE_DISCONNECTED:
-               while (!pDevice->AbortRequest && ((pDevice->State == t_Device::AcquirePaused) ||
-                                                 (pDevice->State == t_Device::VerifyPaused )) )
+               while (!pDevice->Error.Abort() && ((pDevice->GetState() == t_Device::AcquirePaused) ||
+                                                  (pDevice->GetState() == t_Device::VerifyPaused )) )
                   msleep (THREADREAD_CHECK_CONNECTED_SLEEP);
                break;
 
@@ -380,36 +410,40 @@ void t_ThreadRead::run (void)
                LOG_ERROR ("[%s] Unexpected return code from ThreadReadBlock", QSTR_TO_PSZ(pDevice->LinuxDevice))
                CHK_EXIT (rc)
          }
-      } while (!ThreadReadEOF(pDevice) && !pDevice->AbortRequest);
+      } while (!ThreadReadEOF(pDevice) && !pDevice->Error.Abort());
 
-      if (pDevice->AbortRequest)
+      if (pDevice->Error.Abort())
          break;
 
-      if (pDevice->State == t_Device::Acquire)
-           BytesRead     = pDevice->CurrentReadPos;
-      else BytesVerified = pDevice->CurrentReadPos;
+      if (pDevice->GetState() == t_Device::Acquire)
+           BytesRead     = pDevice->CurrentReadPos.Get();
+      else BytesVerified = pDevice->CurrentReadPos.Get();
 
-      if (pDevice->State == t_Device::Verify)
+      if (pDevice->GetState() == t_Device::Verify)
       {
          if (CalcHashMD5)    CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5DigestVerifySrc   ))
+         if (CalcHashSHA1)   CHK_EXIT (HashSHA1Digest   (&HashContextSHA1  , &pDevice->SHA1DigestVerifySrc  ))
          if (CalcHashSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256DigestVerifySrc))
          LOG_INFO ("[%s] Source verification completed", QSTR_TO_PSZ (pDevice->LinuxDevice))
          break;
       }
 
       if (CalcHashMD5)    CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5Digest   ))
+      if (CalcHashSHA1)   CHK_EXIT (HashSHA1Digest   (&HashContextSHA1  , &pDevice->SHA1Digest  ))
       if (CalcHashSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256Digest))
 
-      if ((pDevice->Acquisition.VerifySrc) ||
-          (pDevice->Acquisition.VerifyDst))
+      if ((pDevice->Acquisition1.VerifySrc) ||
+          (pDevice->Acquisition1.VerifyDst) ||
+          (pDevice->Acquisition2.VerifyDst && pDevice->Duplicate))
       {
-         pDevice->State = t_Device::Verify;
+         pDevice->SetState (t_Device::Verify);
          pDevice->StartTimestampVerify = QDateTime::currentDateTime();
       }
 
-      if (pDevice->Acquisition.VerifySrc)
+      if (pDevice->Acquisition1.VerifySrc)
       {
          (void) close (pDevice->FileDescSrc);
+         pDevice->FileDescSrc = t_Device::FileDescEmpty;
          LOG_INFO ("[%s] All data has been read, now re-reading for source verification", QSTR_TO_PSZ (pDevice->LinuxDevice))
          pBlocks = &BlocksVerified;
 //         if (CONFIG (UseSeparateHashThread))
@@ -429,14 +463,13 @@ void t_ThreadRead::run (void)
       if (rcf)
       {
          LOG_INFO ("[%s] Unexpected error on close, errno = %d", QSTR_TO_PSZ(pDevice->LinuxDevice), errno)
-         pDevice->AbortReason  = t_Device::ThreadReadFileError;
-         pDevice->AbortRequest = true;
+         pDevice->Error.Set(t_Device::t_Error::ReadError);
       }
    }
 
    LOG_INFO ("[%s] Read thread exits now (%Ld blocks read, %llu bytes)", QSTR_TO_PSZ (pDevice->LinuxDevice), BlocksRead, BytesRead)
-   if (pDevice->Acquisition.VerifySrc)
-   LOG_INFO ("[%s]   Source verification: %Ld blocks read, %llu bytes)", QSTR_TO_PSZ(pDevice->LinuxDevice), BlocksVerified, BytesVerified)
+   if (pDevice->Acquisition1.VerifySrc)
+      LOG_INFO ("[%s]   Source verification: %Ld blocks read, %llu bytes)", QSTR_TO_PSZ(pDevice->LinuxDevice), BlocksVerified, BytesVerified)
    CHK_EXIT (SetDebugMessage ("Exit run function"))
 }
 
