@@ -9,7 +9,7 @@
 //  Module:         Thread for writing data.
 // ****************************************************************************
 
-// Copyright 2008, 2009, 2010, 2011 Guy Voncken
+// Copyright 2008, 2009, 2010, 2011, 2012, 2013, 2014 Guy Voncken
 //
 // This file is part of guymager.
 //
@@ -44,11 +44,12 @@ class t_ThreadWrite: public t_Thread
 
    public:
       t_ThreadWrite ();
-      t_ThreadWrite (t_pDevice pDevice, bool *pSlowDownRequest);
+      t_ThreadWrite (t_pAcquisition pAcquisition, t_pFifo *ppFifo, bool *pSlowDownRequest);
      ~t_ThreadWrite ();
 
       APIRET GetpFileHandle    (void **ppHandle);
       APIRET ReleaseFileHandle (void);
+      int    Instance          (void);
    private:
       APIRET GetpFileHandle0   (void **ppHandle);
 
@@ -69,6 +70,8 @@ class t_ThreadWrite: public t_Thread
       t_ThreadWriteLocal *pOwn;
 };
 
+typedef t_ThreadWrite *t_pThreadWrite;
+
 // ------------------------------------
 //             Error codes
 // ------------------------------------
