@@ -200,6 +200,52 @@ QString UtilSizeToHuman (unsigned long long Size, bool SI, int FracLen, int Unit
    return SizeHuman;
 }
 
+t_UtilMem::t_UtilMem()
+{
+   oSize = 0;
+   poMem = NULL;
+}
+
+t_UtilMem::t_UtilMem (unsigned long long Size)
+{
+   oSize = Size;
+   poMem = malloc (Size);
+}
+
+t_UtilMem::~t_UtilMem()
+{
+   if (poMem)
+      free(poMem);
+};
+
+void* t_UtilMem::GetpMem (void)
+{
+   return poMem;
+};
+
+unsigned long long t_UtilMem::Size(void)
+{
+   return oSize;
+}
+
+int t_UtilMem::Set (const char *pStr)
+{
+   size_t StrLen = strlen (pStr) +1;
+   int    ret    = 0;
+
+   if (StrLen > (oSize))
+   {
+      StrLen = oSize;
+      ret    = -1;
+   }
+   if (StrLen)
+   {
+      memcpy (poMem, pStr, StrLen-1);
+      ((char*)poMem)[StrLen] ='\0';
+   }
+   return ret;
+}
+
 
 APIRET UtilInit (void)
 {   
