@@ -46,6 +46,22 @@ APIRET UtilGetFreeSpace (const QString &DirPath, quint64 *pFreeBytes);
 
 QString UtilSizeToHuman (unsigned long long Size, bool SI=true, int FracLen=-1, int UnitThreshold=-1);
 
+class t_UtilMem
+{
+   public:
+      t_UtilMem ();
+      t_UtilMem (unsigned long long Size);
+     ~t_UtilMem ();
+
+      unsigned long long Size     (void);
+      void               *GetpMem (void);
+      int                 Set     (const char *pStr);
+
+   private:
+      unsigned long long  oSize;
+      void              *poMem;
+};
+
 
 APIRET UtilInit   (void);
 APIRET UtilDeInit (void);
